# Django library.
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ImproperlyConfigured


# Constants
# ------------------------------------------------------------------

FIFTY_MEGABYTES = 50 * 1024 * 1024
VALID_CONTENT_TYPES = 'image/jpeg', 'image/png', 'image/svg+xml'
NINETY_DPI = 90
IM_WIDTH = 500
IM_HEIGHT = 500
LATENCY = 500

# ------------------------------------------------------------------


def _mdx(var, default):
    """
    Adds "MARXDOWX_" to the requested variable and retrieves its value
    from settings or returns the default.

    :param var: Variable to be retrieved.
    :type var: str
    :param default: Default value if the variable is not defined.
    :return: Value corresponding to 'var'.
    """
    try:
        return getattr(settings, 'MARKDOWNX_' + var, default)
    except ImproperlyConfigured:
        # To handle the auto-generation of documentations.
        return default


# Markdownify
# --------------------
MARKDOWNX_MARKDOWNIFY_FUNCTION = _mdx('MARKDOWNIFY_FUNCTION', 'markdownx.utils.markdownify')

MARKDOWNX_SERVER_CALL_LATENCY = _mdx('SERVER_CALL_LATENCY', LATENCY)


# Markdown extensions
# --------------------
MARKDOWNX_MARKDOWN_EXTENSIONS = _mdx('MARKDOWN_EXTENSIONS', list())

MARKDOWNX_MARKDOWN_EXTENSION_CONFIGS = _mdx('MARKDOWN_EXTENSION_CONFIGS', dict())


# Markdown urls
# --------------------
MARKDOWNX_URLS_PATH = _mdx('URLS_PATH', '/markdownx/markdownify/')

MARKDOWNX_UPLOAD_URLS_PATH = _mdx('UPLOAD_URLS_PATH', '/markdownx/upload/')


# Media path
#  --------------------
MARKDOWNX_MEDIA_PATH = _mdx('MEDIA_PATH', 'markdownx/')


# Image
# --------------------
MARKDOWNX_UPLOAD_MAX_SIZE = _mdx('UPLOAD_MAX_SIZE', FIFTY_MEGABYTES)

MARKDOWNX_UPLOAD_CONTENT_TYPES = _mdx('UPLOAD_CONTENT_TYPES', VALID_CONTENT_TYPES)

MARKDOWNX_IMAGE_MAX_SIZE = _mdx('IMAGE_MAX_SIZE', dict(size=(IM_WIDTH, IM_HEIGHT), quality=NINETY_DPI))

MARKDOWNX_SVG_JAVASCRIPT_PROTECTION = True


# Editor
# --------------------
MARKDOWNX_EDITOR_RESIZABLE = _mdx('EDITOR_RESIZABLE', True)


# ------------------------------------------------
# Translations
# ------------------------------------------------
# This is not called using `_from_settings` as
# it does not need "_MARKDOWNX" prefix.
try:
    LANGUAGES = getattr(
        settings,
        'LANGUAGES',
        (
            ('en', _('English')),
            ('pl', _('Polish')),
            ('de', _('German')),
            ('fr', _('French')),
            ('fa', _('Persian')),
            ('du', _('Dutch'))
        )
    )
except ImproperlyConfigured:
    # To handle the auto-generation of documentations.
    pass



from django.db import models
from django.contrib.admin import site, ModelAdmin

from markdownx.widgets import AdminMarkdownxWidget
from markdownx.models import MarkdownxField

from .models import MyModel


class MyModelAdmin(ModelAdmin):
    formfield_overrides = {
        MarkdownxField: {'widget': AdminMarkdownxWidget},
        models.TextField: {'widget': AdminMarkdownxWidget},
    }


site.register(MyModel, MyModelAdmin)

##
## SHORTER OPTION:
##

# from django.contrib import admin

# from markdownx.admin import MarkdownxModelAdmin

# from .models import MyModel

# admin.site.register(MyModel, MarkdownxModelAdmin)



# -*- coding: utf-8 -*-

import os
BASE_DIR = os.path.dirname(os.path.dirname(__file__))

SECRET_KEY = 'vcpt&w*g54+rlu(le#@_f#y*8^382ubn4uue*xpwrj#upre5ei'

DEBUG = True

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    'markdownx',
    'testapp',
)

MIDDLEWARE = [
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
]

ROOT_URLCONF = 'testapp.urls'

WSGI_APPLICATION = 'testapp.wsgi.application'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}

ALLOWED_HOSTS = ['127.0.0.1', 'localhost']

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

STATIC_URL = '/static/'
STATIC_ROOT = 'testapp/static/'
MEDIA_URL = '/media/'
MEDIA_ROOT = 'testapp/media/'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            'testapp/templates',
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                # Insert your TEMPLATE_CONTEXT_PROCESSORS here or use this
                # list if you haven't customized them:
                'django.contrib.auth.context_processors.auth',
                'django.template.context_processors.debug',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
                'django.template.context_processors.tz',
                'django.contrib.messages.context_processors.messages',
            ],
            'debug': True,
        },
    },
]

MARKDOWNX_MARKDOWN_EXTENSIONS = [
    'markdown.extensions.extra',
]



from django.conf import settings
from django.conf.urls import include, url
from django.conf.urls.static import static
from django.contrib import admin

from testapp.views import (
    IndexTemplateView,
    TestFormView,
    TestCreateView,
)


urlpatterns = [
    url(r'^$', IndexTemplateView.as_view()),
    url(r'^form-view/$', TestFormView.as_view(), name='form_view'),
    url(r'^create-view/$', TestCreateView.as_view(), name='create_view'),
    url(r'^markdownx/', include('markdownx.urls')),
    url(r'^admin/', include(admin.site.urls)),
] + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-05-04 20:21
from __future__ import unicode_literals

from django.db import migrations, models
import markdownx.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='MyModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('markdownx_field1', markdownx.models.MarkdownxField()),
                ('markdownx_field2', markdownx.models.MarkdownxField()),
                ('textfield1', models.TextField()),
                ('textfield2', models.TextField()),
            ],
        ),
    ]



"""
Django MarkdownX
================

|PyPi_Status| |Build_Status| |Format| |Supported_versions_of_Python| |Supported_versions_of_Django| |License|

.. raw:: html

   <iframe src="https://ghbtns.com/github-btn.html?user=neutronX&repo=django-markdownx&type=star&count=true"
   frameborder="0" scrolling="0" width="120px" height="20px"></iframe>
   <iframe src="https://ghbtns.com/github-btn.html?user=neutronX&repo=django-markdownx&type=watch&count=true&v=2"
   frameborder="0" scrolling="0" width="120px" height="20px"></iframe>

------------

Documentations
--------------

Django MarkdownX is a comprehensive Markdown_ plugin built for Django_, the renowned high-level
Python web framework, with flexibility, extensibility, and ease-of-use at its core.


Key features
````````````

* Raw editing.
* Live preview.
* Drag & drop image uploads (automatically stored in the designated location in the *Media* directory).
* Customizable image insertion tag.
* Definition of maximum size for an image.
* Definition of acceptable image formats (PNG, JPEG, SVG).
* Image manipulations (compression, size reduction, cropping, upscaling).
* Pre- and post- text modification.
* Easy template customization, layout modification, and personalization.
* Multiple editors per page.
* Django Admin support.

.. image:: https://github.com/neutronX/django-markdownx/raw/master/django-markdownx-preview.gif?raw=true
   :target: https://github.com/neutronX/django-markdownx
   :align: center
   :alt: django-markdownx preview

.. _Markdown: https://en.wikipedia.org/wiki/Markdown
.. _Django: https://www.djangoproject.com

.. |PyPi_Status| image:: https://img.shields.io/pypi/status/django-markdownx.svg
.. |Build_Status| image:: https://img.shields.io/travis/neutronX/django-markdownx.svg
.. |Format| image:: https://img.shields.io/pypi/format/django-markdownx.svg
.. |Supported_versions_of_Python| image:: https://img.shields.io/pypi/pyversions/django-markdownx.svg
.. |Supported_versions_of_Django| image:: https://img.shields.io/badge/Django-1.8,%201.9,%201.10,%201.11-green.svg
.. |License| image:: https://img.shields.io/pypi/l/django-markdownx.svg

------------
"""

# ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
# Imports
# ~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
# None

# ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
# Documentations
# ~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
__credits__ = 'Adi, Pouria Hadjibagheri'
__copyright__ = 'Copyright 2017'
__license__ = 'BSD'
__maintainer__ = 'Adi, Pouria Hadjibagheri'
__url__ = 'https://github.com/neutronX/django-markdownx'
__version__ = '2.0.20'
__description__ = 'A comprehensive Markdown editor built for Django.'
# ~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-


if __name__ == '__main__':
    raise RuntimeError(
        'MarkdownX is a Django plugin. It is not meant to be run as an stand-alone module.'
    )



from django import forms

from markdownx.fields import MarkdownxFormField
from markdownx.widgets import MarkdownxWidget


class MyForm(forms.Form):
    markdownx_form_field1 = MarkdownxFormField(
        widget=MarkdownxWidget(
            attrs={'class': 'custom-class-markdownx_form_field1'}
        )
    )

    markdownx_form_field2 = MarkdownxFormField(
        widget=MarkdownxWidget(
            attrs={'class': 'custom-class-markdownx_form_field2'}
        )
    )



from django.views.generic.base import TemplateView
from django.views.generic.edit import FormView, CreateView

from testapp.models import MyModel
from testapp.forms import MyForm


class IndexTemplateView(TemplateView):
    template_name = 'index.html'


class TestFormView(FormView):
    template_name = "test_form_view.html"
    form_class = MyForm
    success_url = '/'


class TestCreateView(CreateView):
    template_name = "test_create_view.html"
    model = MyModel
    success_url = '/'
    fields = [
        'markdownx_field1',
        'markdownx_field2',
        'textfield1',
        'textfield2'
    ]



from django import VERSION as DJANGO_VERSION
from django import forms
from django.template.loader import get_template
from django.contrib.admin import widgets
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from .settings import (
    MARKDOWNX_EDITOR_RESIZABLE,
    MARKDOWNX_URLS_PATH,
    MARKDOWNX_UPLOAD_URLS_PATH,
    MARKDOWNX_SERVER_CALL_LATENCY
)


try:
    DEBUG = getattr(settings, 'DEBUG', False)
except ImproperlyConfigured:
    # Documentations work around.
    DEBUG = False


class MarkdownxWidget(forms.Textarea):
    """
    MarkdownX TextArea widget for forms. Markdown enabled version of
    Django "TextArea" widget.
    """

    template_name = 'markdownx/' + ('widget2.html' if DJANGO_VERSION[:2] >= (1, 11) else 'widget.html')

    def get_context(self, name, value, attrs=None):
        """
        Context for the template in Django 1.10 or below.
        """
        if not DJANGO_VERSION[:2] >= (1, 11):
            return super(MarkdownxWidget, self).get_context(name, value, attrs)

        try:
            attrs.update(self.add_markdownx_attrs(attrs))
        except AttributeError:
            attrs = self.add_markdownx_attrs(attrs)

        return super(MarkdownxWidget, self).get_context(name, value, attrs)

    def render(self, name, value, attrs=None, renderer=None):
        """
        Rendering the template and attributes thereof in Django 1.11+.

        .. Note::
            The argument ``renderer`` added in was deprecated in Django 1.11.
        """
        if not DJANGO_VERSION[:2] < (1, 11):
            return super(MarkdownxWidget, self).render(name, value, attrs, renderer)

        attrs = self.build_attrs(attrs, name=name)
        attrs.update(self.add_markdownx_attrs(attrs))

        widget = super(MarkdownxWidget, self).render(name, value, attrs)

        template = get_template(self.template_name)

        return template.render({
            'markdownx_editor': widget,
        })

    @staticmethod
    def add_markdownx_attrs(attrs):
        """
        Setting (X)HTML node attributes.

        :param attrs: Attributes to be set.
        :type attrs: dict
        :return: Dictionary of attributes, including the default attributes.
        :rtype: dict
        """
        if 'class' in attrs.keys():
            attrs['class'] += ' markdownx-editor'
        else:
            attrs.update({
                'class': 'markdownx-editor'
            })

        attrs.update({
            'data-markdownx-editor-resizable': MARKDOWNX_EDITOR_RESIZABLE,
            'data-markdownx-urls-path': MARKDOWNX_URLS_PATH,
            'data-markdownx-upload-urls-path': MARKDOWNX_UPLOAD_URLS_PATH,
            'data-markdownx-latency': MARKDOWNX_SERVER_CALL_LATENCY
        })

        return attrs

    class Media:
        js = {
            'markdownx/js/' + ('markdownx.min.js' if not DEBUG else 'markdownx.js'),
        }


class AdminMarkdownxWidget(MarkdownxWidget, widgets.AdminTextareaWidget):
    """
    MarkdownX TextArea widget for admin. Markdown enabled version of
    Django "TextArea" widget.
    """

    class Media:
        css = {
            'all': {'markdownx/admin/css/' + ('markdownx.min.css' if not DEBUG else 'markdownx.css'), }
        }

        js = {
            'markdownx/js/' + ('markdownx.min.js' if not DEBUG else 'markdownx.js'),
        }



from django.db import models

from markdownx.models import MarkdownxField


class MyModel(models.Model):
    markdownx_field1 = MarkdownxField()
    markdownx_field2 = MarkdownxField()

    textfield1 = models.TextField()
    textfield2 = models.TextField()
